/*
  license GPL v3 ...........

  description of this file:
  a page for viewing a user selected RSS feed ;

*/

import QtQuick 2.0
import QtQuick.XmlListModel 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1
import "./dateutils.js" as DateUtils
import "databasemodule_v2.js" as DB

Item {
    id: articleViewRoot

    signal articleStatusChanged(int tagId, int articleId, string status)
    signal articleFavouriteChanged(var article, string favourite)

    property bool showEmptyPlaceholder: true

    property string rssTitle: ""
    property var rssModel
    property var rssItem: null

    property bool preventIndexChangeHandler: false
    property alias listView: rssListview

    function setNewIndex(rssIndex) {
        rssListview.currentIndex = rssIndex
        rssListview.positionViewAtIndex(rssListview.currentIndex, ListView.Center)
    }

    function setFeed(model, index) {
        /* Setting new model and not-null index will cause two change events instead of one.
         * Settings "preventIndexChangeHandler" to true helps to avoid it.
         */
        if (rssModel != model && index !== 0)
            preventIndexChangeHandler = true
        rssModel = model
        setNewIndex(index)
    }

    function showNextArticle() {
        var index = rssListview.currentIndex + 1
        if (index < rssListview.model.count) {
            rssListview.currentIndex = index
        }
    }

    function showPrevArticle() {
        var index = rssListview.currentIndex - 1
        if (index >= 0) {
            rssListview.currentIndex = index
        }
    }


    //////////////////////////////////////////////      a listview to show the RSS content
    ListView {
        id: rssListview

        property int contentFontSize: optionsKeeper.fontSize()
        property bool contentDarkTheme: optionsKeeper.useDarkTheme()

        width: parent.width
        anchors {
            bottom: parent.bottom
            bottomMargin: units.gu(2)
            top: parent.top
            topMargin: units.gu(2)
        }

        snapMode: ListView.SnapOneItem
        cacheBuffer: 90 // value in pixels, for what?
        // boundsBehavior: Flickable.StopAtBounds
        orientation: ListView.Horizontal
        contentHeight: parent.width * count
        model: rssModel
        delegate: xmlDelegate
        highlightFollowsCurrentItem: true
        highlightRangeMode: ListView.StrictlyEnforceRange
        clip: true

        onCurrentIndexChanged: {
            console.log("ListView onCurrentIndexChanged", currentIndex, preventIndexChangeHandler)

            if (preventIndexChangeHandler) {
                preventIndexChangeHandler = false
                return
            }

            if (rssModel == null || rssModel.get == undefined) {
                console.log("---- Stange behavior ----")
                console.trace()
                return
            }

            if (rssModel.count == 0) // It is normal bevaviour.
                return

            rssItem = rssModel.get(currentIndex)
            rssTitle = rssItem.feed_name

            if (rssItem.status != "1") {
                var dbResult = DB.updateArticleStatus(rssItem.id, "1")
                if (dbResult.rowsAffected == 1) {
                    articleStatusChanged(rssItem.tagId, rssItem.id, "1")
                }
            }
        }

        Label {
            // We want to show it only when view isn't initialized.
            visible: !rssModel && articleViewRoot.showEmptyPlaceholder
            anchors.centerIn: parent
            text: i18n.tr("Select article")
            fontSize: "large"
        }
    }

    //////////////////////////////////////////////      delegate for ListView
    Component {
        id: xmlDelegate

        Flickable {
            id: scrollArea
            objectName: "articleview_flickable"

            clip: true

            width: rssListview.width
            height: rssListview.height

            contentWidth: width
            contentHeight: innerAreaColumn.height

            Column {
                id: innerAreaColumn

                spacing: units.gu(2)
                width: parent.width

                property int mediaDownloadInProgress: 0

                function mediaDownloaded() {
                    mediaDownloadInProgress = mediaDownloadInProgress - 1
                }

                Row {
                    width: parent.width - units.gu(4)
                    height: labelTime.paintedHeight
                    anchors.horizontalCenter: parent.horizontalCenter
                    spacing: units.gu(1)

                    Image {
                        id: imgFavourite
                        anchors.verticalCenter: labelTime.verticalCenter
                        fillMode: Image.PreserveAspectCrop
                        source: Qt.resolvedUrl("./icons_tmp/favorite-selected.svg")
                        sourceSize.height: rssItem == null ? 0 : (rssItem.favourite == "1" ? units.gu(2) : 0)
                        visible: rssItem == null ? false : (rssItem.favourite == "1")
                        smooth: true
                    }

                    Label {
                        id: labelTime
                        text: DateUtils.formatRelativeTime(i18n, pubdate)
                        fontSize: "small"
                        width: parent.width - units.gu(3)
                        wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                    }
                }

                Label {
                    objectName: "articleviewitem_title"
                    fontSize: "large"
                    text: model.title
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width - units.gu(4)
                    wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                }

                Label {
//                    fontSize: "large"
                    color: "grey"
                    text: model.author ? model.author : ""
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width - units.gu(4)
                    wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                    visible: text !== "" && text !== '""'
                }

                Label {
                    text: model.content

                    fontSize: {
                        switch(rssListview.contentFontSize) {
                        case 0:
                            return "small"
                        case 1:
                            return "medium"
                        case 2:
                            return "large"
                        }
                    }
                    color: rssListview.contentDarkTheme ? "lightgray" : "white" // TODO design needed
                    linkColor: "white"
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width - units.gu(4)
                    wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                    clip: true

                    onLinkActivated: Qt.openUrlExternally(model.link)
                }

                Label {
                    id: label_feedname
                    text: rssTitle
                    fontSize: "small"
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width - units.gu(4)
                    wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                }

                Button {
                    id: showMediaButton

                    text: i18n.tr("Show media")
                    anchors.horizontalCenter: parent.horizontalCenter
                    // Everyone can test my "Show media". I want it in v 1.0 too.
                    visible: false // model.media_groups ? true : false

                    onClicked: {
                        visible = false

                        var medias = JSON.parse(rssModel.get(index).media_groups)
                        var len = medias.length

                        for (var i = 0; i < len; i++) {
                            var media = medias[i]

                            for (var j = 0; j < media.contents.length; j++) {
                                var cont = media.contents[j]

                                if (cont.type === "image/jpeg" || cont.type === "image/png" ||
                                        cont.type === "image/jpeg" || cont.type === "image/pjpeg" ||
                                        cont.type === "image/svg+xml" || cont.medium === "image") {
                                    var mediaImg = mediaGroupImageComponent.createObject(innerAreaColumn)

                                    innerAreaColumn.mediaDownloadInProgress++
                                    mediaImg.mediaLoadFinished.connect(innerAreaColumn.mediaDownloaded)

                                    mediaImg.anchors.horizontalCenter = showMediaButton.horizontalCenter
                                    mediaImg.source = cont.url
                                }
                            }
                        }
                    }
                }

                ActivityIndicator {
                    id: mediaDownloadIndicator

                    running: innerAreaColumn.mediaDownloadInProgress > 0
                    visible: running
                    anchors.horizontalCenter: parent.horizontalCenter
                }
            }
        } // Flickable
    } // Component

    Component {
        id: mediaGroupImageComponent

        Image {
            id: mediaGroupImage

            signal mediaLoadFinished

            fillMode: Image.PreserveAspectFit
            width: Math.min(sourceSize.width, articleViewRoot.width - units.gu(4))

            onStatusChanged: {
                if (status == Image.Ready)
                    mediaLoadFinished()
            }
        }
    }
}
