# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013, 2014 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""shorts app autopilot tests."""

from __future__ import absolute_import

from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals, Is, Not

from shorts_app.tests import ShortsAppTestCase
from time import sleep
import unittest
import logging

logger = logging.getLogger(__name__)


_EXTERNAL_FEED_URL = 'http://www.canonical.com/rss.xml'
_EXTERNAL_FEED_TITLE = 'Ubuntu Insights'


class BaseShortsAppTestCase(ShortsAppTestCase):

    def setUp(self):
        super(BaseShortsAppTestCase, self).setUp()

        self.assertThat(
            self.shorts_app.main_view.visible,
            Eventually(Equals(True))
        )
        # wait for any updates to finish before beginning tests
        self._wait_for_refresh()

    def _wait_for_refresh(self):
        try:
            self.assertThat(
                self.shorts_app.main_view.get_network_activity,
                Eventually(NotEquals(None), timeout=5))
            self.assertThat(
                self.shorts_app.main_view.get_network_activity,
                Eventually(Equals(None), timeout=60))
        except:
            self.assertThat(
                self.shorts_app.main_view.get_network_activity,
                Eventually(Equals(None), timeout=60))

    def add_feed_to_new_topic(self, feed_url, feed_title, topic):
        add_feeds_page = self.shorts_app.main_view.go_to_add_feeds()
        # XXX here we are using an external server because we are currently
        # using the Google Feed API, which doesn't let us access local
        # resources. --elopio - 2014-02-26
        add_feeds_page.search_feed(feed_url)
        add_feeds_page.select_results(feed_title)
        choose_topics_page = add_feeds_page.click_next()
        choose_topics_page.add_topic(topic)
        # TODO move this wait to the main view custom proxy object.
        # --elopio - 2014-02-26
        self._wait_for_refresh()


class TestMainWindow(BaseShortsAppTestCase):

    def test_add_feed_to_new_topic(self):
        """Test that adding a feed to a new topic must show it in a new tab."""
        test_feed_url = _EXTERNAL_FEED_URL
        test_feed_title = _EXTERNAL_FEED_TITLE
        test_topic = 'Test topic'
        self.add_feed_to_new_topic(test_feed_url, test_feed_title, test_topic)

        selected_tab_title = self.shorts_app.main_view.get_selected_tab_title()
        self.assertEqual('Test topic', selected_tab_title)

    def _input_new_feed(self, rssFeed):
        inputField = self.shorts_app.main_view.get_append_feed_input_box()
        return self._safe_enter_text(inputField, rssFeed)

    def _input_new_topic(self, topic):
        inputField = self.shorts_app.main_view.get_new_topic_input_box()
        return self._safe_enter_text(inputField, topic)

    def _safe_enter_text(self, inputField, text, timeout=10):
        # poll clicking for focus for lack of better method,
        # due to screen switching animation
        # is there a property we can use instead?
        poll = 0
        while not inputField.focus and poll < timeout:
            self.pointing_device.click_object(inputField)
            sleep(1)
            poll += 1
        self.keyboard.type(text)
        self.assertThat(inputField.text, Eventually(Equals(text)))
        self.keyboard.press_and_release("Enter")
        return inputField

    def _get_topic(self, topicName, timeout=10):
        logger.debug("_get_topic: assert topics")
        self.assertThat(
            lambda: self.shorts_app.main_view.get_feedlist_topic(topicName),
            Eventually(Not(Is([]))))
        logger.debug("_get_topic: grab topic")
        topic = self.shorts_app.main_view.get_feedlist_topic(topicName)
        logger.debug("_get_topic: click topic")

        poll = 0
        while not topic.isExpanded and poll < timeout:
            logger.debug("_get_topic: loop click topic")
            self.pointing_device.click_object(topic)
            sleep(1)
            poll += 1
        return topic

    def test_switch_to_list_view_mode(self):
        """ test switching to list view  mode"""
        shorts_tab = self.shorts_app.main_view.get_shorts_tab()
        self.ensure_grid_view_mode(shorts_tab)
        self.shorts_app.main_view.change_view_mode()
        self.assertThat(shorts_tab.isListMode, Eventually(Equals(True)))

    def test_switch_to_grid_view_mode(self):
        """ test switching to grid view  mode"""
        shorts_tab = self.shorts_app.main_view.get_shorts_tab()
        self.ensure_list_view_mode(shorts_tab)
        self.shorts_app.main_view.change_view_mode()
        self.assertThat(shorts_tab.isListMode, Eventually(Equals(False)))

    def ensure_grid_view_mode(self, current_tab):
        if current_tab.isListMode:
            self.shorts_app.main_view.change_view_mode()

    def ensure_list_view_mode(self, current_tab):
        if not current_tab.isListMode:
            self.shorts_app.main_view.change_view_mode()

    def test_open_listmode_feed_item(self):
        """"test to ensure list mode feed items can be opened"""

        test_feed_url = _EXTERNAL_FEED_URL
        test_feed_title = _EXTERNAL_FEED_TITLE
        test_topic = 'Test topic'

        self.add_feed_to_new_topic(test_feed_url, test_feed_title, test_topic)
        new_topic_tab = self.shorts_app.main_view.get_topic_tab(test_topic)
        self.ensure_list_view_mode(new_topic_tab)

        self.shorts_app.main_view.open_feed_item(test_topic, test_feed_title)
        self.assertEqual(
            self.shorts_app.main_view.get_feed_in_feedlist_title(
                test_topic, test_feed_title),
            self.shorts_app.main_view.get_articleviewitem_title())

    @unittest.skip("Can't see or get dynamic tabs")
    def test_edit_topic(self):
        """test edit topic"""
        test_feed_url = _EXTERNAL_FEED_URL
        test_feed_title = _EXTERNAL_FEED_TITLE
        test_topic = 'Test topic'

        self.add_feed_to_new_topic(test_feed_url, test_feed_title, test_topic)

        # verify we are on the topicName Tab
        editTab = self.shorts_app.main_view.get_topic_tab(test_topic)
        self.assertThat(editTab.visible, Eventually(Equals(True)))

        # open toolbar
        toolbar = self.shorts_app.main_view.open_toolbar()

        # click on edit topics button
        toolbar.click_button("edittopicsbutton")

        # verify we are on topic management page
        topicmanagementpage = \
            self.shorts_app.main_view.get_feed_management_page()
        self.assertThat(
            topicmanagementpage.objectName,
            Eventually(Equals("topicmanagement")))

        # verity Tab is expanded
        editTopic = self.shorts_app.main_view.get_feedlist_topic(test_topic)
        if editTopic.state != "expanded":
            self.pointing_device.click_object(editTopic)

        # select feed
        editFeed = self.shorts_app.main_view.get_feedlist_feed(test_topic)
        self.assertThat(editFeed, NotEquals(None))
        self.pointing_device.click_object(editFeed)

        # verify we are on edit feed page
        editfeedpage = self.shorts_app.main_view.get_edit_feed_page()
        self.assertThat(editfeedpage.visible, Eventually(Equals(True)))

        # change topic from dropdown
        topicValueselector = \
            self.shorts_app.main_view.get_editfeed_topic_valueselector()
        self.pointing_device.click_object(topicValueselector)

        # click on news topic
        topicValueselectorValue = self.shorts_app.main_view. \
            get_editfeed_valueselector_value("Ubuntu")
        self.pointing_device.click_object(topicValueselectorValue)

        # verify topic has changed
        self.assertThat(topicValueselectorValue.text,
                        Eventually(Equals("Ubuntu")))


class ShortsTestCaseWithTopicAndFeed(BaseShortsAppTestCase):

    def setUp(self):
        super(ShortsTestCaseWithTopicAndFeed, self).setUp()
        test_feed_url = _EXTERNAL_FEED_URL
        self.test_feed_title = _EXTERNAL_FEED_TITLE
        self.test_topic = 'Test topic'
        self.add_feed_to_new_topic(
            test_feed_url, self.test_feed_title, self.test_topic)

    def test_remove_feed(self):
        """Test the removal of a feed from the Edit Feeds page."""
        edit_feeds_page = self.shorts_app.main_view.go_to_edit_feeds()
        edit_feeds_page.expand_topic(self.test_topic)
        feeds = edit_feeds_page.get_feeds_in_topic(self.test_topic)
        self.assertIn(self.test_feed_title, feeds)
        edit_feeds_page.delete_feed(self.test_topic, self.test_feed_title)
        feeds = edit_feeds_page.get_feeds_in_topic(self.test_topic)
        self.assertNotIn(self.test_feed_title, feeds)

    def test_remove_topic(self):
        """Test the removal of a topic from the Edit Topics page."""
        edit_feeds_page = self.shorts_app.main_view.go_to_edit_feeds()
        edit_feeds_page.delete_topic(self.test_topic)
        topics = edit_feeds_page.get_topics()
        self.assertNotIn(self.test_topic, topics)
